import pandas as pd
from sklearn import datasets


def get(product):
    d = datasets.load_iris()
    df = pd.DataFrame(d['data'])

    df.columns = d['feature_names']
    df['target'] = d['target']
    df.to_csv(product, index=False)



"""
API for creating scaffolding projects
"""
import itertools
import os
import shutil
import re
from pathlib import Path

try:
    from importlib import resources
except ImportError:
    # python < 3.7
    import importlib_resources as resources

import ploomber_scaffold

import click
from jinja2 import Template


def copy_template(path, package, conda):
    """Copy template files to path
    """
    with resources.path(ploomber_scaffold, 'template') as path_to_template:
        shutil.copytree(path_to_template, path)

    path_to_readme = (path / 'README.md')
    readme = Template(path_to_readme.read_text()).render(package=package,
                                                         conda=conda)
    path_to_readme.write_text(readme)

    if not package:
        simplify(path)

    if conda:
        (path / 'requirements.txt').unlink()
        (path / 'requirements.dev.txt').unlink()
    else:
        (path / 'environment.yml').unlink()
        (path / 'environment.dev.yml').unlink()


def simplify(path):
    to_delete = [
        'MANIFEST.in',
        'setup.cfg',
        'setup.py',
        'tests',
        'versioneer.py',
        Path('src', 'package_name', '_version.py'),
        Path('src', 'package_name', '__init__.py'),
    ]

    for f in to_delete:
        path_to_f = path / f

        if path_to_f.is_file():
            path_to_f.unlink()
        else:
            shutil.rmtree(path_to_f)

    base_path = path / 'src' / 'package_name'

    for f in itertools.chain(base_path.glob('**/*'), base_path.glob('*')):
        f = Path(f)

        if f.is_dir():
            f.mkdir(exist_ok=True, parents=True)
        else:
            target = path / f.relative_to(path / 'src' / 'package_name')
            target.parent.mkdir(exist_ok=True, parents=True)
            shutil.copy(f, target)

    shutil.rmtree(path / 'src')
    Path(path, '.gitattributes').unlink()
    Path(path, 'pipeline.yaml').unlink()

    with resources.path(ploomber_scaffold, 'simple') as path_to_simple:
        shutil.copy(path_to_simple / 'pipeline.yaml', path / 'pipeline.yaml')


def is_valid_package_name(package_name):
    match = re.match(r'^[\w]+$', package_name) or False
    return match and not package_name[0].isnumeric()


def last_part(project_path):
    return project_path.parts[-1]


def request_project_path():
    project_path = Path(input('Enter project name: '))
    pkg_name = last_part(project_path)
    return project_path, pkg_name


def render_template(path, package_name):
    """
    Replace references to "package_name" in the template for the name selected
    by the user
    """
    for dirpath, _, filenames in os.walk(path):
        # replace if file has this extension or is .gitignore
        paths = [
            Path(dirpath, f) for f in filenames if Path(f).suffix in {
                '.py',
                '.md',
                '.yml',
                '.yaml',
                '.sh',
                '.sql',
                '.in',
                '.cfg',
            } or f in {'.gitignore', '.gitattributes'}
        ]

        for p in itertools.chain(paths):
            p = Path(p)
            new = p.read_text().replace('package_name', package_name)
            p.write_text(new)

    pkg_dir = path / 'src' / 'package_name'

    if pkg_dir.exists():
        pkg_dir.rename(path / 'src' / package_name)


def cli(project_path, package=False, conda=False):
    """
    Scaffolds a project

    Parameters
    ----------
    project_path : str
        Project's root folder

    package : bool, default=False
        Whether to create a packaged project (with a setup.py file and
        versioneer) or a simple layout

    conda : bool, default=False
        If True, it adds a conda environment.yml file otherwise
        requirements.txt
    """
    project_path = None if not project_path else Path(project_path)

    click.echo('Enter project name:\n* Alphanumeric\n* Lowercase\n'
               '* Underscores allowed\n'
               '* First character cannot be numeric\n')

    if project_path is None:
        project_path, pkg_name = request_project_path()

        while not is_valid_package_name(pkg_name):
            click.echo(
                f'{pkg_name!r} is not a valid package name, choose another.\n')
            project_path, pkg_name = request_project_path()
    else:
        pkg_name = last_part(project_path)

        if not is_valid_package_name(pkg_name):
            raise ValueError('"%s" is not a valid package identifier, '
                             'choose another.' % pkg_name)

    if project_path.is_dir() and len(os.listdir(project_path)):
        raise click.ClickException(
            f'{str(project_path)!r} is a non-empty directory')
    elif project_path.is_file():
        raise click.ClickException(
            f'{str(project_path)!r} is an existing file')

    copy_template(project_path, package=package, conda=conda)

    render_template(project_path, pkg_name)

    if package:
        path_pipeline = project_path / 'src' / pkg_name / 'pipeline.yaml'
    else:
        path_pipeline = project_path / 'pipeline.yaml'

    if conda:
        path_deps = project_path / 'environment.yml'
        path_deps_dev = project_path / 'environment.dev.yml'
    else:
        path_deps = project_path / 'requirements.txt'
        path_deps_dev = project_path / 'requirements.dev.txt'

    click.secho(f'\nDone. Pipeline declaration: {path_pipeline!s}\n',
                fg='green')
    click.echo('Next steps:\n')
    click.secho(f'1. Add deployment dependencies to {path_deps!s}\n'
                f'2. Add development dependencies to {path_deps_dev!s}\n'
                f'3. Move to {project_path!s}/ and setup the environment'
                ' with: ploomber install')



import io
import re
import ast
from glob import glob
from os.path import basename
from os.path import dirname
from os.path import join
from os.path import splitext

from setuptools import find_packages
from setuptools import setup

_version_re = re.compile(r'__version__\s+=\s+(.*)')

with open('src/ploomber_scaffold/__init__.py', 'rb') as f:
    VERSION = str(
        ast.literal_eval(
            _version_re.search(f.read().decode('utf-8')).group(1)))


def read(*names, **kwargs):
    return io.open(join(dirname(__file__), *names),
                   encoding=kwargs.get('encoding', 'utf8')).read()


DEV = [
    'ploomber',
    'twine',
    'flake8',
    'yapf',
    'pytest',
    'invoke',
    'ipython',
    'nox',
    # ipython not yet compatible with the latest version of jedi
    'jedi==0.17.2',
    # to make this work in Python < 3.7
    'importlib_resources;python_version<"3.7"',
]

setup(
    name='ploomber-scaffold',
    version=VERSION,
    description=None,
    long_description=None,
    author=None,
    author_email=None,
    url='https://github.com/ploomber/ploomber-scaffold',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    include_package_data=True,
    py_modules=[splitext(basename(path))[0] for path in glob('src/*.py')],
    zip_safe=False,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    install_requires=['jinja2'],
    extras_require={
        'dev': DEV,
    },
)
